import re, sys, time, os
import fileinput
from subprocess import call, STDOUT


def parse_linkchecker_output(linkchecker_output):
    pattern = re.compile(r"^URL\s+\`(?P<url>.*?)\'$\n" +
                         r"(^Name\s+\`(?P<name>.*?)\'$\n)?" +
                         r"^Parent URL\s+(?P<parent>.*?)$\n"+
                         r".*?^Result\s+(?P<result>.*?)$"
                         , flags=re.M + re.S)
    de_quote = re.compile(r"\`(.*)\'")

    def read_one(block):
        return {
                "url":block.group("url"),
                "name":block.group("name"),
                "parent_url":block.group("parent"),
                "message":de_quote.sub(r'\1', block.group(0))}

    return {str(x.group("url")):read_one(x) for x in pattern.finditer(linkchecker_output)}.values()


def bad_link(link):
    with open(os.devnull, 'w') as FNULL:
        return call(["linkchecker", "-r0", link["url"]], stdout=FNULL, stderr=STDOUT) != 0


def filter_out_good_links(links, filter_function):
    if links: time.sleep(30)
    return filter(filter_function, links)


def main():
    result = reduce(
            filter_out_good_links,
            [bad_link] * 3,
            parse_linkchecker_output(
                ''.join(fileinput.input())))
    print("\n")
    print("-------------------------------------------------")
    print("----------=========FINAL RESULT========----------")
    print("-------------------------------------------------")
    for r in result:
        print(r["message"])
        print
    if result: sys.exit(1)




'''
Setup script.
To install lizard:
[sudo] python setup.py install
'''

from setuptools import setup, find_packages
import os

def install(appname):

    try:
        with open(os.path.join(os.path.dirname(__file__), 'README.md')) as fobj:
            readme = fobj.read()
    except IOError:
        readme = "helper for linkchecker"

    setup(
          name = appname,
          version = "0.2.11",
          description = '''A tool for retrying linkchecker''',
          long_description =  readme,
          url = 'https://github.com/terryyin/linkchecker-tryer',
          download_url='https://pypi.python.org/linkcheck-retryer/',
          license='MIT',
          platforms='any',
          classifiers = ['Development Status :: 5 - Production/Stable',
                     'Intended Audience :: Developers',
                     'Intended Audience :: End Users/Desktop',
                     'License :: Freeware',
                     'Operating System :: POSIX',
                     'Operating System :: Microsoft :: Windows',
                     'Operating System :: MacOS :: MacOS X',
                     'Topic :: Software Development :: Quality Assurance',
                     'Programming Language :: Python',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3.2',
                     'Programming Language :: Python :: 3.3'],
          py_modules = ['linkchecker_tryer'],
          author = 'Terry Yin',
          author_email = 'terry@odd-e.com',
          scripts = ['linkchecker-tryer']
          )

if __name__ == "__main__":
    import sys
    install('linkchecker-tryer')
